@@ -20,6 +20,20 @@ class TextContent(Content):
             acontenttype = ContentType("text","plain")
         Content.__init__(self, acontenttype, lambda:value.encode("utf8"))
 
+def fixTestCase(test):
+    if not hasattr(test, 'id'):
+        def idfunc(*args): # pylint: disable-msg=W0613
+            if hasattr(test, 'context'):
+                cont = test.context
+                if hasattr(cont, '__module__'):
+                    return cont.__module__+"."+cont.__name__
+                elif hasattr(cont, '__name__'):
+                    return cont.__name__
+                else:
+                    return str(test.context)
+            return str(test)
+        test.id = idfunc
+
 class SubunitTestResult(TestProtocolClient):
     def __init__(self, stream, descriptions, config=None,
                  errorClasses=None, 
@@ -36,6 +50,7 @@ class SubunitTestResult(TestProtocolClient):
         self.stream = stream #this is to make multiprocess plugin happy
         self.isTop = kwargs.get("isTop", False)
         self.useDetails = kwargs.get("useDetails", False)
+        self._wassuccess = True
         TestProtocolClient.__init__(self, stream)
     
     def _getArgs(self, test, err):
@@ -45,17 +60,7 @@ class SubunitTestResult(TestProtocolClient):
         else:
             error = err
             details = None
-        
-        if not hasattr(test, 'id'):
-            def idfunc(*args): # pylint: disable-msg=W0613
-                if hasattr(test, 'context'):
-                    cont = test.context
-                    if hasattr(cont, '__module__'):
-                        return cont.__module__+"."+cont.__name__
-                    else:
-                        return str(test.context)
-                return str(test)
-            test.id = idfunc
+
         return error, details
 
     #properly assign an id() function to nose.failure.Failure (caused
@@ -74,41 +79,57 @@ class SubunitTestResult(TestProtocolClient):
                 return test.address()[1]
             test.id = newid #test.__str__
 
+    #in the case of import module failure, startTest is not called by
+    #nose runner, we have to detect that case and call it manually in
+    #addError to have wellformed subunit output
+    def startTest(self, test):
+        test._subunit_started = True
+        TestProtocolClient.startTest(self, test)
+        
     #modified from nose/result.addError
     def addError(self, test, error): # pylint: disable-msg=W0221
         """Overrides normal addError to add support for
         errorClasses. If the exception is a registered class, the
         error will be added to the list for that class, not errors.
         """
-        stream = self._stream #getattr(self, '_stream', None)
+        
+        fixTestCase(test)
+        #manually call startTest if it's not already called
+        if not getattr(test, '_subunit_started', False):
+            self.startTest(test)
+
         ecls, evt, tbk = error # pylint: disable-msg=W0612
 
         # pylint: disable-msg=W0612
         for cls, (storage, label, isfail) in self.errorClasses.items():
             if isclass(ecls) and issubclass(ecls, cls):
-                if isfail:
-                    test.passed = False
+                if not isfail:
+                    reason = _exception_detail(evt)
+                    if reason and self.useDetails:
+                        details = {"reason":TextContent(reason)}
+                        reason = None
+                    else:
+                        details = None
 
-                # Might get patched into a streamless result
-                if stream is not None:
-                    if not isfail:
-                        reason = _exception_detail(evt)
-                        if reason and self.useDetails:
-                            details = {"reason":TextContent(reason)}
-                            reason = None
-                        else:
-                            details = None
-
-                        self._addNonFailOutcome(label.lower(), test, 
-                          reason=reason, details=details)
-                return
+                    self._addNonFailOutcome(label.lower(), test, 
+                        reason=reason, details=details)
+                    return
+        self._wassuccess = False
         error, details = self._getArgs(test, error)
+        test.passed = False
         TestProtocolClient.addError(self, test, error, details=details)
 
     def addFailure(self, test, error): # pylint: disable-msg=W0221
+        self._wassuccess = False
+        #TestProtocolClient does not call TestResult.addFailure
+        test.passed = False
+        fixTestCase(test)
         error, details = self._getArgs(test, error)
         TestProtocolClient.addFailure(self, test, error, details=details)
 
+    def wasSuccessful(self):
+        return self._wassuccess
+
     def _addNonFailOutcome(self, outcome, test, reason=None, details=None):
         """Report a non-failure error (such as skip)"""
         if reason is None:
